//
//  PlayerView.swift
//  Demos
//
//

import PRESTOPlay
import CastlabsApple
#if !os(tvOS)
import CastlabsChromecast
import CastlabsDownloader
#endif
import CastlabsSubtitles
import CastlabsThumbnails
import CastlabsYoubora
import AVFoundation
import YouboraLib
import SwiftUI

struct PlayerView: View {
    // describe the demo configuration
    let demo: Demo
    // create player instance
    let player: PlayerAPI = PRESTOPlaySDK.shared.player()

    @State private var position: Double = 0
    @State private var thumbnailsVisible: Bool = false
    @State private var thumbnail: UIImage = UIImage()
    
    var body: some View {
        VStack {
            PlayerUIView(
                demo: demo,
                player: player,
                position: $position,
                thumbnailsVisible: $thumbnailsVisible,
                thumbnail: $thumbnail
            )
            .onTapGesture {
                player.playbackState == .playing ? player.pause() : player.play()
            }
            if thumbnailsVisible {
                Image(uiImage: thumbnail)
                    .resizable()
                    .frame(maxWidth: 140.0, maxHeight: 80.0)
            }
            #if !os(tvOS)
            Slider(value: $position, in: 0...100) { editing in
                guard !editing else { return }
                /// this is only a simple demo, please use seekable timeranges to seek
                /// demo present how can thumbnails be implemented
                let value = position / 100.0 * player.getDuration()
                let time = CMTime(seconds: value, preferredTimescale: 1000)
                player.seek(time)
            }
            .padding(15.0)
            #endif
        }
        .onDisappear {
            player.stop()
        }
        #if !os(tvOS)
        .navigationBarTitle(Text(demo.title), displayMode: .inline)
        #endif
    }
}

class PlayerOwnerView: UIView {
    let positionUpdate: (_ position: Double) -> Void
    let thumbnailUpdate: (_ thumbnail: UIImage) -> Void

    var player: PlayerAPI
    var subtitles: SubtitlesAPI?
    var thumbnails: ThumbnailsAPI?
    var analytics: AnalyticsAPI?

    override func layoutSubviews() {
        super.layoutSubviews()
        player.update(size: frame.size)
    }

    init(_ demo: Demo, _ player: PlayerAPI,
         positionCallback: @escaping (Double) -> Void,
         thumbnailsCallback: @escaping (UIImage) -> Void) {
        self.player = player
        self.positionUpdate = positionCallback
        self.thumbnailUpdate = thumbnailsCallback
        super.init(frame: .zero)

        guard let url = demo.url else { return }
        guard let contentURL = URL(string: url) else { return }

        let config = PlayerConfiguration(with: contentURL)
        if let drm = demo.drmConfiguration {
            config.drmType = .drmToday
            config.drmSystem = .fairplay
            config.drmConfiguration = drm
        }
        player.load(config: config)

        self.player.onState = { previous, state in
            if let error = state.playerError {
                print("Error \(error)")
            }

            if state.playbackState == .playing {
                self.positionUpdate(Double(state.currentTime / state.duration) * 100.0)
            }

            if let thumb = self.thumbnails?.getThumbnail(
                atTimestamp: CMTime(seconds: state.currentTime, preferredTimescale: 1)) {

                if let image = UIImage.init(data: thumb.imageData) {
                    DispatchQueue.main.async {
                        self.thumbnailUpdate(image)
                    }
                }
            }

            if state.playbackState == .ready {
                if let subtitlesUrl = demo.sideloadedSubtitle {
                    self.addSubtitles(subtitlesUrl)
                }

                if let thumbnailUrl = demo.sideloadedThumbnail {
                    self.addThumbnails(thumbnailUrl)
                }

                if let youbora = demo.youboraEnabled, youbora {
                    self.addYoubora()
                }
            }
        }

        player.attach(to: layer)
        player.open(autoplay: true)
    }

    func addSubtitles(_ subtitlesUrl: String) {
        subtitles = PRESTOPlaySDK.shared.subtitles(for: player)

        guard let url = URL(string: subtitlesUrl) else {
            print("unable to parse subtitles url")
            return
        }

        let subtitlesTrack = TextTrack(
            id: UUID().uuidString,
            format: .web_vtt,
            isPlatformRendered: false,
            label: "sideloaded",
            language: "en",
            sourceUrl: url)

        subtitles?.addTextTrack(subtitlesTrack, completionHandler: { error in
            if let error { print(error) } else { print("subtitles track loaded") }
            DispatchQueue.main.async {
                self.player.setTextTrack(subtitlesTrack)
            }
        })
    }

    func addThumbnails(_ thumbnailUrl: String) {
        thumbnails = PRESTOPlaySDK.shared.thumbnails(for: player)

        guard let url = URL(string: thumbnailUrl) else {
            print("unable to parse thumbnails url")
            return
        }

        thumbnails?.loadThumbnailsFrom(webVTTtrack: url, completionHandler: { error in
            if let error { print(error) } else { print("thumbnails loaded") }
        })
    }

    func addYoubora() {
        analytics = PRESTOPlaySDK.shared.analytics(for: player, .youbora)
        let youboraMetadata = YouboraMetadata(
            live: false,
            assetId: "assetId"
        )
        youboraMetadata.options = YBOptions()
        youboraMetadata.options?.adCustomDimension1 = "test"
        youboraMetadata.options?.contentTitle = "test"
        analytics?.metadata = youboraMetadata
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

struct PlayerUIView: UIViewRepresentable {
    let demo: Demo
    let player: PlayerAPI

    @Binding var position: Double
    @Binding var thumbnailsVisible: Bool
    @Binding var thumbnail: UIImage

    func makeUIView(context: Context) -> UIView {
        return PlayerOwnerView(demo, player) {
            position = $0
        } thumbnailsCallback: {
            thumbnailsVisible = true
            thumbnail = $0
        }
    }

    func updateUIView(_ uiView: UIView, context: Context) { }
}

struct PlayerView_Previews: PreviewProvider {
    static var previews: some View {
        PlayerView(demo: Demo(
            title: "HLS Playback",
            description: "Basic HLS playback demo"
        ))
    }
}
